<?php

// If this file is called directly, abort.
defined( 'WPINC' ) || die( 'Well, get lost.' );


/**
 * Class WP_Persistent_Login_Settings_Premium
 *
 * @since 2.0.0
 */
class WP_Persistent_Login_Settings_Premium extends WP_Persistent_Login_Settings {

    public $post;
    public $message;
    public $type;
    public $message_key;
    public $type_key;

    /**
	 * Initialize the class and set its properties.
	 *
	 * We register all our common hooks here.
	 *
	 * @since  2.0.0
	 * @access public
	 *
	 * @return void
	 */
	public function __construct() {

        parent::__construct();        
		
	}



    /**
     * handle_settings
     *
     * @return void
     */
    public function handle_settings() {

        $post_data = $this->post;
        $method = sanitize_text_field($post_data['wppl_method']);
        
        if( method_exists($this, $method) ) {

            if( wp_verify_nonce($post_data[$method.'_nonce'], $method.'_action') ) {

                $this->$method($post_data);
            
            } else {
            
                // redirect the user and notify an error
                $redirect_url = esc_url_raw($post_data['_wp_http_referer']);
                $code = 302;
                $message = 'Failed to update settings, your security nonce was invalid.';
                $type = 'error';
                $this->redirect_with_message($redirect_url, $code, $message, $type);
            
            }
        
        }

    }


        
    /**
     * update_general_settings
     *
     * @param  array $post_data
     * @return void
     */
    protected function update_general_settings($post_data) {

        // update control for hiding dashboard stats
        $this->update_dashboard_stats($post_data);

        // update control for plugin specific logic
        $this->update_duplicate_sessions($post_data);

        // update persistent login roles
        $this->update_login_user_roles($post_data);

        // update login time
        $this->update_login_time_limit($post_data);

        // update hide/show remember me box
        $this->update_hide_remember_me($post_data);

    }
    

    /**
	 * update_active_login_settings
	 *
	 * @param  mixed $post_data
	 * @return void
	 */
	protected function update_active_login_settings($post_data) {

		// update user preferences for active logins
		$this->update_limit_active_logins($post_data);

        // update the roles that active logins are applied to
        $this->update_active_login_roles($post_data);

        // update the number of active logins that are allowed
        $this->update_active_logins_allowed($post_data);

        // update the page to redirect to when login limit is reached
        $this->update_maximum_logins_redirect($post_data);
		
	}

    /**
     * get_premium_settings
     *
     * @return array
     */
    protected function get_premium_settings() {

        $premium_settings = get_option('persistent_login_options_premium');
        return $premium_settings;

    }
        

    /**
     * update_premium_settings
     *
     * @param  array $settings
     * @return bool
     */
    private function update_premium_settings_option($settings) {

        return update_option('persistent_login_options_premium', $settings);

    }

    
    /**
     * get_login_user_roles
     *
     * @return array
     */
    public function get_login_user_roles() {

        $premium_settings = $this->get_premium_settings();
        if( isset($premium_settings['roles']) ) {
            $user_roles = $premium_settings['roles'];
        } else {
            $user_roles = array();
        }
        
        return $user_roles;

    }
    
    
    /**
     * update_login_user_roles
     *
     * @param  array $post_data
     * @return bool
     */
    private function update_login_user_roles($post_data) {

        if( isset($post_data['roles']) ) : 
							    
            $roles = $post_data['roles'];

            $premium_settings = $this->get_premium_settings();
            
            // if there are settings, add them
            foreach( $roles as $role ) :
                $role = sanitize_text_field( $role );
                $role_options[] = $role;
            endforeach;

            $premium_settings['roles'] = $role_options;

            // update cookie options and return true|false
            $update_settings = $this->update_premium_settings_option($premium_settings);
            
            return $update_settings;

        endif;

    }



    /**
     * get_login_time_limit
     *
     * @return array
     */
    public function get_login_time_limit() {

        $premium_settings = $this->get_premium_settings();

        if( isset($premium_settings['cookieTime']) ) {

            $login_time_limit = $premium_settings['cookieTime'];

            $days = floor($login_time_limit/86400);
            $hours = floor(($login_time_limit-$days*86400)/(60 * 60));
            $mins = floor(($login_time_limit-($days*86400+$hours*3600))/60);

        } else {

            $days = floor(YEAR_IN_SECONDS/86400);
            $hours = floor((YEAR_IN_SECONDS-$days*86400)/(60 * 60));
            $mins = floor((YEAR_IN_SECONDS-($days*86400+$hours*3600))/60);

        }

        return array(
            'days' => $days,
            'hours' => $hours,
            'mins' => $mins
        );

    }
    

    /**
     * update_login_time_limit
     *
     * @param  array $post_data
     * @return bool
     */
    private function update_login_time_limit($post_data) {

        if( isset($post_data['days']) || isset($post_data['hours']) || isset($post_data['mins']) ) {

            // sanitize the inputs fro the user
            $days = filter_var($post_data['days'], FILTER_SANITIZE_NUMBER_INT);
            $hours = filter_var($post_data['hours'], FILTER_SANITIZE_NUMBER_INT);
            $mins = filter_var($post_data['mins'], FILTER_SANITIZE_NUMBER_INT);

            $premium_settings = $this->get_premium_settings();

            // Days
            if( !empty($days) ) : 
                $days = floatval($days) * 86400; // 86,400 seconds per day
            else : 
                $days = 0;
            endif;
            
            // Hours
            if( !empty($hours) ) : 
                $hours = floatval($hours) * 3600; // 3,600 seconds per hour
                else : 
                    $hours = 0;
                endif;
                
            // Mins
            if( !empty($mins) ) : 
                $mins = floatval($mins) * 60; // 1 second per second...
                else : 
                    $mins = 0;
                endif;
            
            
            // add the individual settings up and update the premium_settings array
            $login_time_limit = $days + $hours + $mins;
            $premium_settings['cookieTime'] = $login_time_limit;
             
            // update cookie options and return true|false
            $update_settings = $this->update_premium_settings_option($premium_settings);
            return $update_settings;

        }

    }


    /**
     * get_hide_remember_me
     *
     * @return string
     */
    public function get_hide_remember_me() {

        $premium_settings = $this->get_premium_settings();

        if( isset($premium_settings['hideRememberMe']) ) {
            $hide_remember_me = $premium_settings['hideRememberMe'];
        } else {
            $hide_remember_me = '0';
        }
        
        return $hide_remember_me;

    }
    

    /**
     * update_hide_remember_me
     *
     * @param  array $post_data
     * @return bool
     */
    private function update_hide_remember_me($post_data) {

        if( isset($post_data['hideRememberMe']) ) :
            $hide_remember_me = sanitize_text_field($post_data['hideRememberMe']);
        else :
            $hide_remember_me = '0';
        endif;
        
        // fetch the current premium settings
        $premium_settings = $this->get_premium_settings();

        // update the hide remember me setting to the new option
        $premium_settings['hideRememberMe'] = $hide_remember_me;

        // update cookie options and return true|false
        $update_settings = $this->update_premium_settings_option($premium_settings);
        return $update_settings;

    }



    /**
     * get_active_logins_allowed
     *
     * @return string
     */
    public function get_active_logins_allowed() {

        $premium_settings = $this->get_premium_settings();

        if( isset($premium_settings['activeLoginsAllowed']) ) {
            $active_logins_allowed = $premium_settings['activeLoginsAllowed'];
        } else {
            $active_logins_allowed = '1';
        }
       
        return $active_logins_allowed;

    }


  
    /**
     * update_active_logins_allowed
     *
     * @param  array $post_data
     * @return bool
     */
    public function update_active_logins_allowed($post_data) {

        $premium_settings = $this->get_premium_settings();
        $premium_settings['activeLoginsAllowed'] = $post_data['activeLoginsAllowed'];

        // update cookie options and return true|false
        $update_settings = $this->update_premium_settings_option($premium_settings);
            
        return $update_settings;

    }

    
    /**
     * update_maximum_logins_redirect
     *
     * @param  array $post_data
     * @return bool
     */
    public function update_maximum_logins_redirect($post_data) {

        if( isset($post_data['maximumLoginsRedirect']) ) {
            $premium_settings = $this->get_premium_settings();
            $premium_settings['maximumLoginsRedirect'] = $post_data['maximumLoginsRedirect'];
    
            // update cookie options and return true|false
            $update_settings = $this->update_premium_settings_option($premium_settings);
                
            return $update_settings;
        }

    }

    
    /**
     * get_maximum_logins_redirect
     *
     * @return int
     */
    public function get_maximum_logins_redirect() {

        $premium_settings = $this->get_premium_settings();

        if( isset($premium_settings['maximumLoginsRedirect']) ) {
            $maximum_logins_redirect = intval($premium_settings['maximumLoginsRedirect']);
        } else {
            $maximum_logins_redirect = NULL;
        }

        return $maximum_logins_redirect;
    }

    
    /**
     * get_active_login_roles
     *
     * @return array
     */
    public function get_active_login_roles() {

        $premium_settings = $this->get_premium_settings();

        if( isset($premium_settings['activeLoginRoles']) ) {
            $active_login_roles = $premium_settings['activeLoginRoles'];
        } else {
            $active_login_roles = array();
        }
       
        return $active_login_roles;

    }


    /**
     * update_active_login_roles
     *
     * @param  array $post_data
     * @return bool
     */
    private function update_active_login_roles($post_data) {

        if( isset($post_data['activeLoginRoles']) ) : 
							    
            $roles = $post_data['activeLoginRoles'];

            $premium_settings = $this->get_premium_settings();
            
            // if there are settings, add them
            foreach( $roles as $role ) :
                $role = sanitize_text_field( $role );
                $role_options[] = $role;
            endforeach;

            $premium_settings['activeLoginRoles'] = $role_options;

            // update cookie options and return true|false
            $update_settings = $this->update_premium_settings_option($premium_settings);
            
            return $update_settings;

        endif;

    }




    /**
	 * persistent_login_options_display
	 *
	 * @since 2.0.0
	 * @return string
	 */
	public function persistent_login_options_display() {

		//$settings = new WP_Persistent_Login_Settings_Premium();

		if( isset($_GET['view']) ) :
			
			// updated db version
			if( $_GET['view'] == 'update' ) :
				$message = __('WordPress Persistent Login has been updated to the latest database version!', 'wp-persistent-login' );
				$class = 'notice updated';
				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) ); 
			endif;
		
		endif;
		

		?>
	
		<div class="wrap">
			
			<h1><?php _e('WordPress Persistent Login', 'wp-persistent-login' ); ?></h1>
			<h2 style="float: left; margin-top: 0;"><?php _e('Premium Plan', 'wp-persistent-login' ); ?></h2>
			
			<div style="float: right;">
				<p>
					<a href="<?php echo WPPL_ACCOUNT_PAGE; ?>" class="button">
						<?php _e('My Account', 'wp-persistent-login' ); ?>
					</a>
					<a href="<?php echo WPPL_UPGRADE_PAGE; ?>" class="button">
						<?php _e('Change my plan', 'wp-persistent-login' ); ?>
					</a>
					<a href="<?php echo WPPL_SUPPORT_PAGE; ?>" class="button">
						<?php _e('Premium Support', 'wp-persistent-login' ); ?>
					</a>
				</p>
			</div>
			<div class="clear"></div>	


			<div class="main-content" style="width: calc(100% - 270px); float: left;">

				<?php
					$default_tab = null;
					$tab = isset($_GET['tab']) ? $_GET['tab'] : $default_tab;
				?>
				<nav class="nav-tab-wrapper">
					<a 
						href="<?php echo WPPL_SETTINGS_PAGE; ?>" 
						class="nav-tab <?php if($tab===null):?>nav-tab-active<?php endif; ?>"
					>
						Dashboard
					</a>
					<a 
						href="<?php echo WPPL_SETTINGS_PAGE; ?>&tab=persistent-login" 
						class="nav-tab <?php if($tab==='persistent-login'):?>nav-tab-active<?php endif; ?>"
					>
						Persistent Login
					</a>
					<a 
						href="<?php echo WPPL_SETTINGS_PAGE; ?>&tab=active-logins" 
						class="nav-tab <?php if($tab==='active-logins'):?>nav-tab-active<?php endif; ?>"
					>
						Active Logins
					</a>
				</nav>
		
				<div class="tab-content">
					<?php if( !isset($tab) ) : ?>

						<h1>Dashboard</h1>
						<p>
							<?php 
								_e(
									'Persistent login will keep all users logged in automatically. Forever.', 
									 'wp-persistent-login' ); 
							?>
						</p>
						<?php $this->output_login_count_meta_box(); ?>

					
					<?php elseif( $tab === 'persistent-login' ) : ?>
					
						<h1>
							<?php 
								_e(
									'Persistent Login Settings', 
									 'wp-persistent-login' ); 
							?>
						</h1>
						<p>
							<?php 
								_e(
									'Control how users are kept logged into your website over time.', 
									 'wp-persistent-login' ); 
							?>
						</p>
						<form method="POST">
					
							<input type="hidden" name="wppl_method" value="update_general_settings" />
							<?php wp_nonce_field( 'update_general_settings_action', 'update_general_settings_nonce' ); ?>
							
							<table class="form-table">
								<tbody>   

									<!-- roles -->
									<tr style="border-bottom: 1px solid #dfdfdf;">
										
										<th><br/>
											Persistent login roles<br/>
										</th>
										
										<td>
											<br/>
											<?php 
												global $wp_roles; 
												$persistent_login_roles = $this->get_login_user_roles();
											?>
											<?php foreach( $wp_roles->roles as $key=>$value ) : ?>
												<label style="width: 250px; max-width: 45%; display: inline-block; margin-bottom: 10px;">
													<input 
														name="roles[]" id="roles" type="checkbox" value="<?php echo $key; ?>" class="regular-checkbox" 
														<?php
															if( !empty($persistent_login_roles) ) : 
																echo ( in_array($key, $persistent_login_roles) ) ? 'checked'  : ''; 
															endif;
														?>  
													/> 
													<?php echo $value['name']; ?>
												</label>
												&nbsp;&nbsp;&nbsp;
											<?php endforeach; ?>
											<p class="description">
												Which roles should use persistent login? Unchecking an option will logout users with that role.
											</p>
											<br/>
										</td>
									</tr>
									<!-- END roles -->


									<!-- logged in time -->						
									<?php
										// Fetch current login time settings
										$login_time_limit = $this->get_login_time_limit();
										$days = $login_time_limit['days'];
										$hours = $login_time_limit['hours'];
										$mins = $login_time_limit['mins'];
									?>
									<tr style="border-bottom: 1px solid #dfdfdf;">
									
										<th><br/>
											Login time limit
										</th>
										
										<td>
											<br/>
											<label style="width: 200px; max-width: 45%; display: inline-block; margin-bottom: 10px;">
												<input name="days" id="days" type="number" class="small-text" max="365"
													value="<?php echo ($days) ? $days : '0'; ?>" 
												/> 
												Days &nbsp; &nbsp;
											</label>
											<label style="width: 200px; max-width: 45%; display: inline-block; margin-bottom: 10px;">
												<input name="hours" id="hours" type="number" class="small-text" max="8760"
													value="<?php echo ($hours) ? $hours : '0'; ?>" 
												/> 
												Hours &nbsp; &nbsp;
											</label>
											<label style="width: 200px; max-width: 45%; display: inline-block; margin-bottom: 10px;">
												<input name="mins" id="mins" type="number" class="small-text" max="525600"
													value="<?php echo ($mins) ? $mins : '0'; ?>" 
												/> 
												Minutes &nbsp; &nbsp;
											</label><br/>
											<p class="description">
												How long should Persistent Login keep users logged in for? <strong>Maximum of 365 days</strong>.
											</p>
											<p class="description">
												<small>
													If a user doesn't visit your website within this time, they will have to login again. If they visit before this time, they will automatically be logged in.
												</small>
											</p>
										</td>
									</tr>
									<!-- END loggedin time -->


									<!-- toggle show/hide rememberme box -->
                                    <?php $hide_remember_me = $this->get_hide_remember_me(); ?>
									<tr style="border-bottom: 1px solid #dfdfdf;">
										<th><br/> 
											Remember me boxes<br/>
										</th>
										<td>
											<br/>
											
											<label style="width: auto; display: inline-block;">
												<input 
													name="hideRememberMe" id="hideRememberMe" type="checkbox" value="1" 
													class="regular-checkbox" <?php echo ($hide_remember_me === '0' || $hide_remember_me === NULL ) ? '' : 'checked'; ?>
												/>
												Hide all 'Remember Me' checkboxes from users
											</label>
											<br/>
											<p class="description">
												<small>
													(stop users unchecking the 'Remember Me' box)
												</small>
											</p>
										</td>
									</tr>
									<!-- END toggle show/hide rememberme box -->
							
							
									<!-- dashboard at a glance screen -->	
                                    <?php $hide_dashboard_stats = $this->get_dashboard_stats(); ?>					
									<tr style="border-bottom: 1px solid #dfdfdf;">
									
										<th><br/>
											<?php _e('Dashboard panel options', 'wp-persistent-login' ); ?><br/>
										</th>
										<td>
											<br/>
											<label style="width: auto; display: inline-block;">
												<input 
													name="hidedashboardstats" id="hidedashboardstats" type="checkbox" value="1" 
													class="regular-checkbox" <?php echo ($hide_dashboard_stats !== '0') ? 'checked' : ''; ?>
												/> 
												<?php _e('Hide \'At a glance\' dashboard stats', 'wp-persistent-login' ); ?>
											</label><br/>
											<br/>
										</td>
									</tr>
									<!-- END dashboard at a glance screen -->		
							
							
									<!-- allow duplicate sessions -->
									<?php $duplicate_sessions = $this->get_duplicate_sessions(); ?> 
									<tr style="border-bottom: 1px solid #dfdfdf;">
										<th>
											<br/> 
											<?php _e('Duplicate sessions', 'wp-persistent-login' ); ?><br/>
										</th>
										<td>
											<br/>
											<label style="width: auto; display: inline-block;">
												<input 
													name="duplicateSessions" id="duplicateSessions" type="checkbox" value="1" 
													class="regular-checkbox" <?php echo ($duplicate_sessions === '0' || $duplicate_sessions === NULL ) ? '' : 'checked'; ?>
												/>
												<?php _e('Allow duplicate sessions', 'wp-persistent-login' ); ?>
											</label><br/>
											<p class="description">
												<small>
													<?php _e('(select if you\'re having trouble staying logged in on multiple devices)', 'wp-persistent-login' ); ?>
												</small>
											</p>
										</td> 
									</tr>
									<!-- END allow duplicate sessions -->
							     
						
								</tbody>
							</table>
							<p class="submit">
                                <input 
									type="submit" name="submit" id="submit" class="button button-primary" 
									value="<?php _e('Save Persistent Login Settings', 'wp-persistent-login' ); ?>"
								>
							</p>
						</form>
					
					<?php elseif( $tab === 'active-logins' ) : ?>
					
						<h1>
							<?php 
								_e(
									'Active Login Settings', 
									 'wp-persistent-login' ); 
							?>
						</h1>
						<p>
							<?php 
								_e(
									'Control how many active logins users can have at any one time.', 
									 'wp-persistent-login' ); 
							?>
						</p>

                        <form method="POST">
					
							<input type="hidden" name="wppl_method" value="update_active_login_settings" />
							<?php wp_nonce_field( 'update_active_login_settings_action', 'update_active_login_settings_nonce' ); ?>

                            <table class="form-table">
                                <tbody>

                                    <!-- enable active login limit -->
                                    <?php $limit_active_logins = $this->get_limit_active_logins(); ?> 
                                    <tr style="border-bottom: 1px solid #dfdfdf;">
                                        <th>
                                            <?php _e('Enable active logins', 'wp-persistent-login' ); ?><br/>
                                        </th>
                                        <td>
                                            <label style="width: auto; display: inline-block; margin-bottom: 20px;">
                                                <input 
                                                    name="limitActiveLogins" id="limitActiveLogins" type="checkbox" value="1" 
                                                    class="regular-checkbox" <?php echo ($limit_active_logins === '0' || $limit_active_logins === NULL ) ? '' : 'checked'; ?>
                                                />
                                                <?php _e('Enable active logins limit', 'wp-persistent-login' ); ?>
                                            </label><br/>
                                        </td>
                                    </tr>
                                    <!-- END enable active login limit -->

                                    <!-- enable active login limit -->
                                    <?php $active_logins_allowed = $this->get_active_logins_allowed() ?: '1'; ?> 
                                    <tr style="border-bottom: 1px solid #dfdfdf;">
                                        <th>
                                            <?php _e('Active logins limit', 'wp-persistent-login' ); ?><br/>
                                        </th>
                                        <td>
                                            <label style="width: auto; display: inline-block; margin-bottom: 0px;">
                                                <?php _e('Allow users to have', 'wp-persistent-login' ); ?>
                                                <input name="activeLoginsAllowed" id="activeLoginsAllowed" 
                                                type="number" class="small-text" min="1"
                                                    value="<?php echo $active_logins_allowed; ?>" 
                                                /> 
                                                <?php _e('active login(s)', 'wp-persistent-login' ); ?> &nbsp; &nbsp;
                                            </label><br/>
                                            <p class="description">
                                                <small>
                                                    <?php _e('(This limit will only apply if "Enable active logins limit" is checked)', 'wp-persistent-login' ); ?>
                                                </small>
                                            </p>
                                        </td>
                                    </tr>
                                    <!-- END enable active login limit -->

                                    <!-- limit reached -->
                                    <?php 
                                        $limit_reached_logic = $this->get_limit_reached_logic(); 
                                        $limit_reached_options = array(
                                            array(
                                                'label' => __('Automatically end the oldest active login for the user.', 'wp-persistent-login' ),
                                                'value' => 'automatic'
                                            ),
                                            array(
                                                'label' => __('Allow users to select which active logins to end.', 'wp-persistent-login' ),
                                                'value' => 'manual'
                                            ),
                                            array(
                                                'label' => __( 'Block new logins if active login limit reached.', 'wp-persistent-login' ),
                                                'value' => 'block'
                                            )
                                        );
                                    ?>
                                    <tr>
                                        <th>
                                            <?php _e('Limit reached logic', 'wp-persistent-login' ); ?><br/>
                                        </th>
                                        <td>
                                            <?php foreach( $limit_reached_options as $option ) : ?>
                                                <label style="width: auto; display: inline-block; margin-bottom: 10px;">
                                                    <input 
                                                        name="activeLoginLogic" 
                                                        id="<?php echo $option['value']; ?>" 
                                                        type="radio" 
                                                        value="<?php echo $option['value']; ?>" 
                                                        class="regular-radio js-maximum-logins-logic" 
                                                        <?php echo ( $limit_reached_logic === $option['value'] ) ? ' checked' : ''; ?>
                                                    /> 
                                                    <?php echo $option['label']; ?>
                                                </label><br/>
                                            <?php endforeach; ?>
                                        </td>
                                    </tr>
                                    <!-- END limit reached -->


                                    <?php
                                        $selected_option = $this->get_maximum_logins_redirect();
                                        $pages_query = new WP_Query(array(
                                            'post_type' => 'page',
                                            'post_status' => 'publish',
                                            'posts_per_page' => -1,
                                            'orderby' => 'name',
                                            'order' => 'ASC'
                                        ));
                                        if( $limit_reached_logic === 'manual' ) {
                                            $class = '';
                                        } else {
                                            $class = 'hide';
                                        }
                                    ?>
                                    <!-- maximum logins redirect -->
                                    <tr class="js-maximum-logins-redirect <?php echo $class; ?>">
                                        <th>
                                            
                                        </th>
                                        <td style="padding-top: 0;">

                                            <h4><?php _e('Limit reached redirect', 'wp-persistent-login' ); ?></h4>

                                            <p style="margin-bottom: 1rem;">
                                                <?php _e('When the number of active logins reaches the limit set, the user will be redirect to this page to end a session before they can continue to login.', 'wp-persistent-login'); ?>
                                            </p>

                                            <select name="maximumLoginsRedirect" id="maximumLoginsRedirect" <?php echo ($class !== 'hide') ? 'required' : ''; ?>>
                                            <option value="" disabled selected>Please select a page to redirect users</option>
                                                <?php if( $pages_query->have_posts() ) : ?>
                                                    <?php while( $pages_query->have_posts() ) : $pages_query->the_post(); ?>
                                                        <?php 
                                                            if( get_the_ID() === $selected_option ) {
                                                                $selected = 'selected';
                                                            } else {
                                                                $selected = '';
                                                            }
                                                        ?>
                                                        <option value="<?php echo get_the_ID(); ?>" <?php echo $selected; ?>>
                                                            <?php echo get_the_title(); ?>
                                                        </option>
                                                    <?php endwhile; ?>
                                                <?php endif; ?>
                                            </select>
                                            <?php if( $selected_option !== 0 ) : ?>
                                                <br/><br/>
                                                <a href="<?php echo admin_url(); ?>post.php?post=<?php echo $selected_option; ?>&action=edit" target="_blank" class="button">
                                                    <?php _e('Edit selected page', 'wp-persistent-login' ); ?>
                                                </a>
                                                <br/>
                                            <?php endif; ?>

                                            <div style="box-sizing: border-box; margin-top: 10px; padding: 10px 10px 10px 20px; background-color: #dee6ec; border-left: 3px solid #2271b1;">
                                                <p style="padding-bottom: 5px;">
                                                    <?php 
                                                    echo sprintf(
                                                        __('The selected page %s must %s include the %s Maximum Logins Control %s block, or the %s shortcode to work correctly. ', 'wp-persistent-login' ),
                                                        '<strong>',
                                                        '</strong>',
                                                        '<strong>',
                                                        '</strong>',
                                                        '<span style="display: inline-block; margin-top: 0px; padding: 2px 5px; 
                                                        background-color: #ffffff; border: 1px solid #c5c4c0;">[wppl-maximum-logins-control]</span>'
                                                    ); ?>
                                                </p>
                                            </div>

                                        </td>
                                    </tr>
                                    <!-- END maximum logins redirect -->

                                    <!-- active login roles -->
                                    <tr style="border-top: 1px solid #dfdfdf; border-bottom: 1px solid #dfdfdf;">
                                            
                                        <th>
                                            <?php _e('Active login roles', 'wp-persistent-login' ); ?><br/>
                                        </th>
                                        
                                        <td>
                                            <?php 
                                                global $wp_roles; 
                                                $active_login_roles = $this->get_active_login_roles();
                                            ?>
                                            <?php foreach( $wp_roles->roles as $key=>$value ) : ?>
                                                <label style="width: 250px; max-width: 45%; display: inline-block; margin-bottom: 10px;">
                                                    <input 
                                                        name="activeLoginRoles[]" id="activeLoginRoles" type="checkbox" value="<?php echo $key; ?>" class="regular-checkbox" 
                                                        <?php
                                                            if( !empty($active_login_roles) ) : 
                                                                echo ( in_array($key, $active_login_roles) ) ? 'checked'  : ''; 
                                                            endif;
                                                        ?>  
                                                    /> 
                                                    <?php echo $value['name']; ?>
                                                </label>
                                                &nbsp;&nbsp;&nbsp;
                                            <?php endforeach; ?>
                                            <p class="description">
                                                <?php _e('Which roles should the active login limit apply to? Unchecking a role will allow unlimited active logins for that role.', 'wp-persistent-login' ); ?>
                                            </p>
                                            <br/>
                                        </td>
                                    </tr>
                                    <!-- END active login roles -->

                                    <!-- manage active logins -->
                                    <tr style="border-bottom: 1px solid #dfdfdf;">
                                        <th>
                                            <?php _e('Manage active logins', 'wp-persistent-login' ); ?><br/>
                                        </th>

                                        <td>
                                            <h4 style="margin-bottom: 0;"><?php _e('Front End', 'wp-persistent-login' ) ?></h4>
                                            <p><?php _e('Let users manage their active logins on different devices from the front-end using either of the methods below:', 'wp-persistent-login' ); ?></p>
                                
                                            <div style="box-sizing: border-box; margin-top: 10px; padding: 15px 15px 15px 20px; background-color: #dee6ec; border-left: 3px solid #2271b1;">
                                                <p style="margin-top: 0;">
                                                    <?php _e('Add the <strong>Manage Sessions</strong> Block from the Widgets section. ', 'wp-persistent-login' ); ?>
                                                    <?php _e('Or add this shortcode anywhere in the classic editor:', 'wp-persistent-login' ); ?><br/>
                                                    <span style="display: inline-block; margin-top: 2px; padding: 5px; 
                                                    background-color: #ffffff; border: 1px solid #c5c4c0;">
                                                        [wppl-manage-sessions]
                                                    </span>
                                                </p>
                                                            
                                            </div>
                                
                                            <p class="description">
                                                <small>
                                                    <?php _e('Add to any page to allow logged-in users to manage their sessions.', 'wp-persistent-login' ); ?> 
                                                </small>
                                            </p>
                                            <br/>
                                
                                            <hr/>
                                
                                            <h4 style="margin-bottom: 0;">WP Admin</h4>
                                
                                            <p>
                                                <?php _e('You and your users can manage active logins from their profile page in the dashboard.', 'wp-persistent-login' ); ?>
                                            </p>
                                            
                                            <p>
                                                <?php _e('Click on a user to view their profile and scroll down to the Sessions heading to manage their active logins.', 'wp-persistent-login' ); ?>
                                            </p>
                                            <br/>
                                            
                                            <p>
                                                <a href="<?php echo admin_url(); ?>users.php" class="button">
                                                    <?php _e('View All Users', 'wp-persistent-login' ); ?>
                                                </a>
                                                &nbsp;<?php _e('or', 'wp-persistent-login' ); ?>&nbsp;
                                                <a href="<?php echo admin_url(); ?>profile.php#sessions" class="button">
                                                    <?php _e('Visit Your Profile', 'wp-persistent-login' ); ?>
                                                </a>
                                                <?php _e('to see your sessions', 'wp-persistent-login' ); ?>
                                            </p>
                                            
                                            <br/>
                                        </td>
                                    </tr>
                                    <!-- END manage sessions -->


                                </tbody>
                            </table>
                            <p class="submit">
                                <input 
                                    type="submit" name="submit" id="submit" class="button button-primary" 
                                    value="<?php _e('Save Active Login Settings', 'wp-persistent-login' ); ?>"
                                >
                            </p>

                        </form>
							
					<?php endif; ?>	
				</div>
		


			</div>


			<div class="postbox-container sidebar" style="max-width: 250px; float: right;">
				<div class="metabox-holder"> 
					<div class="postbox">
						<div class="inside">
							<h3 style="margin-top: 1rem; cursor: auto;">
                                <?php _e('Want a new feature?', 'wp-persistent-login' ); ?>
                            </h3>
							<p>
                                <?php _e("If you'd like to see a new feature on WordPress Persistent Login, just request it by clicking the button below and <strong>choose the Feature Request option</strong>.", 'wp-persistent-login' ); ?>
                            </p>
							<a href="<?php echo admin_url(); ?>options-general.php?page=wp-persistent-login-contact" class="button">
								<?php _e('Request a Feature', 'wp-persistent-login'); ?>
							</a>
						</div>
					</div>
				</div>
			</div>

			<div style="clear:both; display: block;"></div>
							
		</div>

		<style>
			.tab-content {
				padding: 2.5%;
			}
			.form-table tbody tr td {
				padding-top: 25px;
				padding-bottom: 20px;
			}
            .hide {
                display: none;
            }
            @media all and ( max-width: 1100px ) {
				.main-content {
					width: 100% !important;
					float: none !important;
				}
				.sidebar {
					float: none !important;
				}
			}
		</style>
        <script>
            var activeLoginLogicoptions = document.querySelectorAll('[name="activeLoginLogic"]');
            var activeLoginRedirect = document.querySelector('[name="maximumLoginsRedirect"]');
            var maximumLoginsRedirect = document.querySelector('.js-maximum-logins-redirect');

            activeLoginLogicoptions.forEach(function(activeLoginLogic, index) {
                activeLoginLogic.addEventListener('change', function() {

                if( activeLoginLogic.getAttribute('value') === 'manual' ) {
                    maximumLoginsRedirect.classList.remove('hide');
                    activeLoginRedirect.required = 'required';
                } else {
                    maximumLoginsRedirect.classList.add('hide');
                    activeLoginRedirect.removeAttribute('required');
                }
                });
            });
            
        </script>
		<?php
	
		
	} // end persistent_login_options_display	


}